-- 2021-04-22 License CC0 (Creative Commons license) (c) darkfrei

function create_particle (data)
	local particle = {}
	particle.radius = data and data.radius or 50
	particle.color = {
		0.5+0.5*(particle.radius%10)/10, 
		0.5+0.5*(particle.radius%20)/20, 
		0.5+0.5*(particle.radius%50)/50, 
	}
	particle.x = math.random (particle.radius, width - particle.radius)
	particle.y = math.random (particle.radius, height - particle.radius)
	local vel = 200+100*math.random ()
	local angle = math.random () * 2 * math.pi
	particle.vx =  vel * math.cos(angle)
	particle.vy = -vel * math.sin(angle)
	particle.ax, particle.ay = 0, 0
	table.insert(particles, particle)
end

function create_grid ()
	grid = {}
	local size = grid_size
	local max_x, max_y = width-size, height-size
	for x = 0, max_x, size do
		grid[x]={}
		for y = 0, max_y, size do
			local a = 132/255
			local b = 10/255
			local c = (x/size+y/size)%2
			grid[x][y]={color = {a+b*c, a+b*c, a+b*c}, size = size}
--			print ('x: '..x..' y:'..y)
		end
	end
end

function love.load()
	local ddwidth, ddheight = love.window.getDesktopDimensions( display )
	if ddheight > 1080 then -- for video capture
		print('ddheight: ' .. ddheight)
		love.window.setMode(1920, 1080, {resizable=false, borderless=true})
	else
		love.window.setMode(ddwidth, ddheight-200, {resizable=true, borderless=false})
	end
	
	grid_size = 50
	
	width, height = love.graphics.getDimensions( )
	width, height = math.floor(width/grid_size)*grid_size, math.floor(height/grid_size)*grid_size
	
	
	particles = {}
	create_grid ()
	for i = 1, 10 do
		create_particle ({radius = math.random (10, 50)})
	end
	
end

function update_accel (particle, dt)
	local ddx, ddy = particle.ax*dt, particle.ay*dt
	particle.vx=particle.vx+ddx
	particle.vy=particle.vy+ddx
end

function update_speed (particle, dt)
	local dx, dy = particle.vx*dt, particle.vy*dt
	particle.x=particle.x+dx
	particle.y=particle.y+dy
	
	if particle.x < particle.radius  then -- continuous collision detection
		particle.x = particle.x - (particle.radius-particle.x)/dx
		particle.vx = - particle.vx
	elseif particle.x > (width - particle.radius) then
		particle.x = particle.x + (width - particle.radius-particle.x)/dx
		particle.vx = - particle.vx
	end
	if particle.y < particle.radius then
		particle.y = particle.y - (particle.radius-particle.y)/dy
		particle.vy = - particle.vy
		elseif particle.y > (height - particle.radius) then
		particle.y = particle.y  + (height - particle.radius-particle.y)/dy
		particle.vy = - particle.vy
	end
end

function update_particle (particle, dt)
	update_accel (particle, dt)
	update_speed (particle, dt)
end
 
function update_grid (dt)
	-- erase all old data
	for x,ys in pairs (grid) do
		for y, g in pairs (ys) do
			g.particles = {}
		end
	end
	
	for i, particle in pairs (particles) do
		local x1 = math.floor ((particle.x-particle.radius)/grid_size)*grid_size
		
		local x2 = math.floor ((particle.x+particle.radius)/grid_size)*grid_size
		
		local y1 = math.floor ((particle.y-particle.radius)/grid_size)*grid_size
		local y2 = math.floor ((particle.y+particle.radius)/grid_size)*grid_size
		
		for x = x1, x2, grid_size do
			for y = y1, y2, grid_size do
			
				if grid[x] and grid[x][y] then
					table.insert (grid[x][y].particles, particle)
				end	
			end
		end
	end
end


function love.update(dt)
	if dt > 0.1 then dt = 0.1 end
	for i, particle in pairs (particles) do
		update_particle (particle, dt)
	end
	
	update_grid (dt)
	
end


function love.draw()
	for x, ys in pairs (grid) do
		for y, g in pairs (ys) do
			love.graphics.setColor(g.color)
--			love.graphics.setColor({0.1, 0.1, 0.1})
			if #g.particles == 1 then
				love.graphics.setColor(1,1,0)
			elseif #g.particles == 2 then
				love.graphics.setColor(0,1,0)
			elseif #g.particles > 2 then
				love.graphics.setColor(1,0,0)
			end
			love.graphics.rectangle('fill', x, y, g.size, g.size)
		end
	end
	for i, particle in pairs (particles) do
		love.graphics.setColor(particle.color)
		love.graphics.circle('fill', particle.x, particle.y, particle.radius)
	end
end

function love.keypressed(key, scancode, isrepeat)
   if key == "escape" then
      love.event.quit()
   end
end